-------------------------------------------------------------------------
----                                                                 ----
---- Suska boot loader unit                                          ----
----                                                                 ----
---- This file is part of the SUSKA ATARI clone project.             ----
---- http://www.experiment-s.de                                      ----
----                                                                 ----
---- Description:                                                    ----
---- This boot loader is written for flash memory type M29W800       ----
---- or similar.                                                     ----
---- The boot loader unit provides the programming of the opera-     ----
---- ting system flash ROM during a boot sequence. The working       ----
---- principle of the boot loader is as follows:                     ----
---- Enter the boot loader by pressing the RESET_INn and then        ----
---- RESET_COREn button. Then release first the RESET_COREn          ----
---- button then the RESET_INn button. The BOOT_LED begins blinking  ----
---- for about 5 seconds. If the RESET_INn button is pressed again   ----
---- during this time, the boot loader routine begins with the       ----
---- erasing of the flash memory. If the RESET_INn is not pressed    ----
---- within these five seconds, the boot loader routine finishes     ----
---- without any action, the system is ready to boot normally.       ----
---- Once entered the flash erase routine, the boot loader erases    ----
---- the complete flash memory, then enters the request address      ----
---- routine in which a start address for the write cyle is re-      ----
---- quested via the SPI interface. This state must be acknow-       ----
---- ledged by the SPI host. If not so, the boot loader hangs and    ----
---- ends up only by a new system reset. The start address is 16     ----
---- bit wide and mactches the upper bits of the flash memory        ----
---- here: (A19 ... A4). After the start address is acknowledged,    ----
---- the boot loader enters the write section and proceeds after     ----
---- it with the read section. The read or write section is          ----
---- controlled by request acknowledge and ends up in a time out     ----
---- of about 1 second means if a request is not acknowledged        ----
---- after the time out, the current state is finished. After the    ----
---- read state finishes, the system is in normal operation.         ----
---- During the write section the boot loader requests data from     ----
---- the SPI interface via BOOT_REQ. If the data is received and     ----
---- the acknowledge BOOT_ACK_S is asserted the data is written to   ----
---- the flash chip and the next data is requested. The address      ----
---- of the flash is incremented automatically. It starts at zero    ----
---- and ends up at the maximum address for the chip which is        ----
---- given by the number of the address lines. The data written      ----
---- is always 16 bit wide which results in shifting in 16 bits      ----
---- for each data via the SPI interface. If the maximum address     ----
---- is acknowledged, the address counter rolls over and starts      ----
---- again with address zero. Be aware, that writing a flash         ----
---- twice with different data normally results in corrupted data.   ----
---- The read process works similar with reading out 16 bits         ----
---- after the request is asserted.                                  ----
---- During the shift operation via the SPI interface, the most      ----
---- significant bit is first shifted out and in.                    ----
---- Be aware, that the data in the shift register is sampled on     ----
---- the negative clock edge and the SPI_CLK should be '0' when      ----
---- inactive.                                                       ----
----                                                                 ----
---- 20071220: This boot loader is prepared to handle 8 bit SPI data ----
---- and 16 bit Flash data and address. The boot controller manages  ----
---- this by requesting two bytes per Request for each flash word or ----
---- for the address offset.                                         ----
----                                                                 ----
---- Author(s):                                                      ----
---- - Wolfgang Foerster, wf@experiment-s.de; wf@inventronik.de      ----
----                                                                 ----
-------------------------------------------------------------------------
----                                                                 ----
---- Copyright (C) 2007 Wolfgang Foerster                            ----
----                                                                 ----
---- This source file may be used and distributed without            ----
---- restriction provided that this copyright statement is not       ----
---- removed from the file and that any derivative work contains     ----
---- the original copyright notice and the associated disclaimer.    ----
----                                                                 ----
---- This source file is free software; you can redistribute it      ----
---- and/or modify it under the terms of the GNU Lesser General      ----
---- Public License as published by the Free Software Foundation;    ----
---- either version 2.1 of the License, or (at your option) any      ----
---- later version.                                                  ----
----                                                                 ----
---- This source is distributed in the hope that it will be          ----
---- useful, but WITHOUT ANY WARRANTY; without even the implied      ----
---- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR         ----
---- PURPOSE. See the GNU Lesser General Public License for more     ----
---- details.                                                        ----
----                                                                 ----
---- You should have received a copy of the GNU Lesser General       ----
---- Public License along with this source; if not, download it      ----
---- from http://www.gnu.org/licenses/lgpl.html                      ----
----                                                                 ----
-------------------------------------------------------------------------
-- 
-- Revision History
-- 
-- Revision 2K7A  2007/01/22 WF
--   Initial Release.
-- Revision 2K7B  2007/12/21 WF
--   Fixed a bug in the boot state machine.
--   Changes to meet 16bit SPI / 8bit MCU interface.
--   Changes to get the controller working with the
--   M29W800 flash.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity WF_FLASHBOOT is
	port(
		CLK				: in bit; -- Use 16MHz.
		PLL_LOCK		: in bit;
		RESET_COREn		: in bit;
		RESET_INn		: in bit;
		RESET_OUTn		: out bit;
		
		-- Bus control:
		ROM_CEn			: in bit;
		
		-- Data and address bus:
		ADR_OUT			: out std_logic_vector(19 downto 0);
		ADR_EN			: out bit;
		DATA_IN			: in std_logic_vector(15 downto 0);
		DATA_OUT		: out std_logic_vector(15 downto 0);
		DATA_EN			: out bit;
		
		-- Flash interface:
		FLASH_RDY		: in bit;
				
		FLASH_RESETn	: out bit;
		FLASH_WEn		: out bit;
		FLASH_OEn		: out bit;
		FLASH_CEn		: out bit;

		-- Microcontroller interface:		
		SPI_CLK			: in bit;
		SPI_DIN			: in bit;
		SPI_DOUT		: out bit;
		
		BOOT_ACK		: in bit;
		BOOT_REQ		: out bit;
		
		-- Status:
		BOOT_LED		: out bit
	);
end entity WF_FLASHBOOT;

architecture BEHAVIOR of WF_FLASHBOOT is
type FLASH_CMD_A is array (1 to 2, 1 to 6) of bit_vector(11 downto 0);
type FLASH_CMD_D is array (1 to 2, 1 to 6) of bit_vector(7 downto 0);
constant FLASH_CMDS_A : FLASH_CMD_A := 
	((x"555", x"2AA", x"555", x"555", x"2AA", x"555"),  -- Erase entire chip.
	 (x"555", x"2AA", x"555", x"000", x"000", x"000")); -- Program.
constant FLASH_CMDS_D : FLASH_CMD_D := 
	((x"AA", x"55", x"80", x"AA", x"55", x"10"),  -- Erase entire chip.
	 (x"AA", x"55", x"A0", x"00", x"00", x"00")); -- Program.
type BOOT_STATES is (WAIT_PLL_LOCK, ATTENTION, WAIT_KEYRELEASE,
                    ACTIVATE, IDLE, FLASH_ERASE, ADR_REQ, BOOT_WR, BOOT_RD);
type ERASE_STATES is (IDLE, FLASH_INIT, WRITE_CMD, READY);
type WRITE_STATES is(IDLE, FLASH_INIT, WRITE_CMD, WAIT_ACK, WRITE_DATA, WRITE_END);
type READ_STATES is(IDLE, WAIT_ACK, SET_ADR, DATA_LOAD);
signal BOOT_STATE		: BOOT_STATES;
signal NEXT_BOOT_STATE	: BOOT_STATES;
signal ERASE_STATE		: ERASE_STATES;
signal NEXT_ERASE_STATE	: ERASE_STATES;
signal WRITE_STATE		: WRITE_STATES;
signal NEXT_WRITE_STATE	: WRITE_STATES;
signal READ_STATE		: READ_STATES;
signal NEXT_READ_STATE	: READ_STATES;
signal ADR_OUT_I		: bit_vector(19 downto 0);
signal DATA_OUT_I		: bit_vector(15 downto 0);
signal D_SHIFTREG		: bit_vector(15 downto 0);
signal ADR_REG			: std_logic_vector(19 downto 0);
signal TIME_5			: boolean;
signal RESET_TIMEOUT	: boolean;
signal ACK_TIMEOUT		: boolean;
signal CMD_PNTR			: integer range 0 to 7;
signal INIT_RDY			: boolean;
signal FLASH_RDY_S		: bit;
signal SPI_CLK_S		: bit;
signal SPI_DIN_S		: bit;
signal BOOT_ACK_S		: bit;
begin
	P_SYNC: process
	-- Input synchronizer.
	begin
		wait until CLK = '0' and CLK' event;
		FLASH_RDY_S <= FLASH_RDY;
		SPI_CLK_S <= SPI_CLK;
		SPI_DIN_S <= SPI_DIN;
		BOOT_ACK_S <= BOOT_ACK;
	end process P_SYNC;

	STATE_REGs: process(RESET_COREn, CLK)
	begin
		if RESET_COREn = '0' then
			BOOT_STATE <= WAIT_PLL_LOCK;
			ERASE_STATE <= IDLE;
			WRITE_STATE <= IDLE;
			READ_STATE <= IDLE;
		elsif CLK = '1' and CLK' event then
			BOOT_STATE <= NEXT_BOOT_STATE;
			ERASE_STATE <= NEXT_ERASE_STATE;
			WRITE_STATE <= NEXT_WRITE_STATE;
			READ_STATE <= NEXT_READ_STATE;
		end if;
	end process STATE_REGs;

	BOOT_STATE_DEC: process(BOOT_STATE, PLL_LOCK, RESET_INn, TIME_5, ERASE_STATE, BOOT_ACK_S, ACK_TIMEOUT)
	begin
		case BOOT_STATE is
			when WAIT_PLL_LOCK => -- PLLs must be locked first.
				if PLL_LOCK = '1' then
					NEXT_BOOT_STATE <= ATTENTION;
				else
					NEXT_BOOT_STATE <= WAIT_PLL_LOCK;
				end if;
			when ATTENTION =>
				if RESET_INn = '1' then -- No boot mode.
					NEXT_BOOT_STATE <= IDLE;
				else -- enter boot mode.
					NEXT_BOOT_STATE <= WAIT_KEYRELEASE;
				end if;
			when WAIT_KEYRELEASE =>
				if RESET_INn = '1' then
					NEXT_BOOT_STATE <= ACTIVATE;
				else
					NEXT_BOOT_STATE <= WAIT_KEYRELEASE;
				end if;
			when ACTIVATE => -- A second RESET_INn enters the Flash routine.
				if RESET_INn = '0' then
					NEXT_BOOT_STATE <= FLASH_ERASE;
				elsif TIME_5 = true then
					NEXT_BOOT_STATE <= IDLE;
				else
					NEXT_BOOT_STATE <= ACTIVATE;
				end if;
			when FLASH_ERASE =>
				if ERASE_STATE = READY then
					NEXT_BOOT_STATE <= ADR_REQ;
				else
					NEXT_BOOT_STATE <= FLASH_ERASE;
				end if;
			when ADR_REQ =>
				if BOOT_ACK_S = '1' then
					NEXT_BOOT_STATE <= BOOT_WR;
				else
					NEXT_BOOT_STATE <= ADR_REQ;
				end if;
			when BOOT_WR =>
				if ACK_TIMEOUT = true then
					NEXT_BOOT_STATE <= BOOT_RD;
				else
					NEXT_BOOT_STATE <= BOOT_WR;
				end if;
			when BOOT_RD =>
				if ACK_TIMEOUT = true then
					NEXT_BOOT_STATE <= IDLE;
				else
					NEXT_BOOT_STATE <= BOOT_RD;
				end if;
			when IDLE =>
				NEXT_BOOT_STATE <= IDLE; -- Bootmode finished, stay here.
		end case;
	end process BOOT_STATE_DEC;

	ERASE_DEC: process(ERASE_STATE, BOOT_STATE, NEXT_BOOT_STATE, INIT_RDY, FLASH_RDY_S, BOOT_ACK_S)
	begin
		case ERASE_STATE is
			when IDLE =>
				if BOOT_STATE /= FLASH_ERASE and NEXT_BOOT_STATE = FLASH_ERASE then
					NEXT_ERASE_STATE <= FLASH_INIT;
				else
					NEXT_ERASE_STATE <= IDLE;
				end if;
			when FLASH_INIT =>
				if INIT_RDY = true then
					NEXT_ERASE_STATE <= READY;
				elsif FLASH_RDY_S = '1' then
					NEXT_ERASE_STATE <= WRITE_CMD;
				else
					NEXT_ERASE_STATE <= FLASH_INIT;
				end if;
			when WRITE_CMD =>
				NEXT_ERASE_STATE <= FLASH_INIT;
			when READY =>
				if BOOT_ACK_S = '0' then -- Must be inactive.
					NEXT_ERASE_STATE <= IDLE;
				else
					NEXT_ERASE_STATE <= READY;
				end if;
		end case;
	end process ERASE_DEC;
	
	WRITE_DEC: process(WRITE_STATE, BOOT_STATE, NEXT_BOOT_STATE, INIT_RDY, FLASH_RDY_S, BOOT_ACK_S, ACK_TIMEOUT, RESET_TIMEOUT)
	begin
		case WRITE_STATE is
			when IDLE =>
				if BOOT_STATE /= BOOT_WR and NEXT_BOOT_STATE = BOOT_WR then
					NEXT_WRITE_STATE <= FLASH_INIT;
				else
					NEXT_WRITE_STATE <= IDLE;
				end if;
			when FLASH_INIT =>
				if INIT_RDY = true then
					NEXT_WRITE_STATE <= WAIT_ACK;
				elsif FLASH_RDY_S = '1' then
					NEXT_WRITE_STATE <= WRITE_CMD;
				else
					NEXT_WRITE_STATE <= FLASH_INIT;
				end if;
			when WRITE_CMD =>
				NEXT_WRITE_STATE <= FLASH_INIT;
			when WAIT_ACK =>
				if ACK_TIMEOUT = true then
					NEXT_WRITE_STATE <= WRITE_END;
				elsif BOOT_ACK_S = '1' and FLASH_RDY_S = '1' then
					NEXT_WRITE_STATE <= WRITE_DATA;
				else
					NEXT_WRITE_STATE <= WAIT_ACK;
				end if;
			when WRITE_DATA =>
				NEXT_WRITE_STATE <= FLASH_INIT;
			when WRITE_END =>
				if RESET_TIMEOUT = true then
					NEXT_WRITE_STATE <= IDLE;
				else
					NEXT_WRITE_STATE <= WRITE_END;
				end if;
		end case;
	end process WRITE_DEC;

	READ_DEC: process(READ_STATE, BOOT_STATE, NEXT_BOOT_STATE, ACK_TIMEOUT, BOOT_ACK_S)
	begin
		case READ_STATE is
			when IDLE =>
				if BOOT_STATE /= BOOT_RD and NEXT_BOOT_STATE = BOOT_RD then
					NEXT_READ_STATE <= WAIT_ACK;
				else
					NEXT_READ_STATE <= IDLE;
				end if;
			when WAIT_ACK =>
				if ACK_TIMEOUT = true then
					NEXT_READ_STATE <= IDLE;
				elsif BOOT_ACK_S = '1' then
					NEXT_READ_STATE <= SET_ADR;
				else
					NEXT_READ_STATE <= WAIT_ACK;
				end if;
			when SET_ADR =>
				-- The SET_ADR is introduced to meet the timing requirements
				-- for the flash device.
				if BOOT_ACK_S = '0' then
					NEXT_READ_STATE <= DATA_LOAD;
				else
					NEXT_READ_STATE <= SET_ADR;
				end if;
			when DATA_LOAD =>
				NEXT_READ_STATE <= WAIT_ACK;
		end case;
	end process READ_DEC;

	LED_BLNK: process
	-- During the ACTIVATE state the LED blinks with
	-- a frequency of 1Hz.
	variable TMP : std_logic_vector(23 downto 0);
	begin
		wait until CLK = '1' and CLK' event;
		case BOOT_STATE is
			when ACTIVATE | FLASH_ERASE | ADR_REQ | BOOT_WR | BOOT_RD =>
				TMP := TMP + '1';
			when others =>
				TMP := (others => '0');
		end case;
		-- 
		case BOOT_STATE is
			when ACTIVATE =>
				BOOT_LED <= To_Bit(TMP(23));
			when FLASH_ERASE =>
				BOOT_LED <= '1';
			when ADR_REQ | BOOT_WR | BOOT_RD =>
				BOOT_LED <= To_Bit(TMP(21));
			when others =>
				BOOT_LED <= '0';
		end case; 
	end process LED_BLNK;
	
	TIMER_5: process
	-- This process provides a delay of 5 seconds.
	variable TMP : std_logic_vector(25 downto 0);
	begin
		wait until CLK = '1' and CLK' event;
		TIME_5 <= false;
		if BOOT_STATE /= ACTIVATE then
			TMP := (others => '0');
		elsif TMP < "11" & x"FFFFFF" then
			TMP := TMP + '1';
		else
			TIME_5 <= true;
		end if;
	end process TIMER_5;

	P_TIMEOUT: process
	-- This process provides a delay of about 1 second
	-- in the WAIT_ACK states, the timeout occurs after
	-- this time. In other states the timeout timer
	-- is disabled
	variable TMP : std_logic_vector(23 downto 0);
	begin
		wait until CLK = '1' and CLK' event;
		ACK_TIMEOUT <= false;
		if WRITE_STATE = WAIT_ACK and TMP < x"FFFFFF" then
			TMP := TMP + '1';
		elsif READ_STATE = WAIT_ACK and TMP < x"FFFFFF" then
			TMP := TMP + '1';
		elsif WRITE_STATE = WAIT_ACK then
			ACK_TIMEOUT <= true;
		elsif READ_STATE = WAIT_ACK then
			ACK_TIMEOUT <= true;
		else
			TMP := (others => '0');
		end if;
	end process P_TIMEOUT;

	P_RESET: process
	-- At the end of a single pulse program mode, some flash
	-- devices require a reset of at least 500ns to switch to
	-- the read mode (not the M29W800). This timer provides 
	-- this delay.
	variable TMP : std_logic_vector(3 downto 0);
	begin
		wait until CLK = '1' and CLK' event;
		RESET_TIMEOUT <= false;
		if WRITE_STATE = WRITE_END and TMP < x"F" then
			TMP := TMP + '1';
		elsif WRITE_STATE = WRITE_END then
			RESET_TIMEOUT <= true;
		else
			TMP := (others => '0');
		end if;
	end process P_RESET;

	SEQ_CNT: process
	-- This process controls during the respective states the
	-- writing of the correct number of commands to the flash device.
	variable CNT : std_logic_vector(2 downto 0);
	begin
		wait until CLK = '1' and CLK' event;
		if ERASE_STATE = WRITE_CMD or WRITE_STATE = WRITE_CMD then
			CNT := CNT - '1';
		elsif BOOT_STATE = ACTIVATE then
			CNT := "110"; -- 6 command words for erasing the chip.
		elsif BOOT_STATE = ADR_REQ then
			CNT := "011"; -- 3 command words for writing data.
		end if;
		--
		case CNT is
			when "000" => INIT_RDY <= true;
			when others => INIT_RDY <= false;
		end case;
		--
		CMD_PNTR <= conv_integer(CNT);
	end process SEQ_CNT;
	
	ADR_COUNTER: process(RESET_COREn, CLK)
	begin
		if RESET_COREn = '0' then
			ADR_REG <= x"00000";
		elsif CLK = '1' and CLK' event then
			if BOOT_STATE = ADR_REQ and BOOT_ACK_S = '1' then
				ADR_REG(19 downto 4) <= To_StdLogicVector(D_SHIFTREG); -- Load the start address.
			elsif WRITE_STATE = WRITE_DATA or READ_STATE = DATA_LOAD then
				ADR_REG <= ADR_REG + '1';
			end if;			
		end if;
	end process ADR_COUNTER;

	SHFT: process(RESET_COREn, CLK)
	-- This is the SPI receiver transmitter shift
	-- register.
	variable LOCK : boolean;
	begin
		if RESET_COREn = '0' then
			D_SHIFTREG <= x"0000";
		elsif CLK = '1' and CLK' event then
			if READ_STATE = DATA_LOAD then
				D_SHIFTREG <= To_BitVector(DATA_IN); -- Load flash data.
			elsif SPI_CLK_S = '0' and LOCK = false then
				D_SHIFTREG <= D_SHIFTREG(14 downto 0) & SPI_DIN_S;
				LOCK := true;
			elsif SPI_CLK_S = '1' then
				LOCK := false;
			end if;			
		end if;
	end process SHFT;

	SPI_DOUT <= D_SHIFTREG(15); -- MSB first out.

	ADR_EN <= '1' when BOOT_STATE = FLASH_ERASE or BOOT_STATE = BOOT_WR or BOOT_STATE = BOOT_RD else '0';
	ADR_OUT_I <=	x"00" & FLASH_CMDS_A(1, CMD_PNTR) when ERASE_STATE = FLASH_INIT else
					x"00" & FLASH_CMDS_A(1, CMD_PNTR) when ERASE_STATE = WRITE_CMD and INIT_RDY = false else
					x"00" & FLASH_CMDS_A(2, CMD_PNTR) when WRITE_STATE = FLASH_INIT else
					x"00" & FLASH_CMDS_A(2, CMD_PNTR) when WRITE_STATE = WRITE_CMD and INIT_RDY = false else To_BitVector(ADR_REG);

	DATA_EN <= '1' when BOOT_STATE = FLASH_ERASE or BOOT_STATE = BOOT_WR else '0';
	DATA_OUT_I <= 	x"00" & FLASH_CMDS_D(1, CMD_PNTR) when ERASE_STATE = FLASH_INIT else
					x"00" & FLASH_CMDS_D(1, CMD_PNTR) when ERASE_STATE = WRITE_CMD and INIT_RDY = false else
					x"00" & FLASH_CMDS_D(2, CMD_PNTR) when WRITE_STATE = FLASH_INIT else
					x"00" & FLASH_CMDS_D(2, CMD_PNTR) when WRITE_STATE = WRITE_CMD and INIT_RDY = false else D_SHIFTREG;
	
	ADR_OUT <= To_StdLogicVector(ADR_OUT_I);
	DATA_OUT <= To_StdLogicVector(DATA_OUT_I);

	BOOT_REQ <= '1' when BOOT_STATE = ADR_REQ else
				'1' when WRITE_STATE = WAIT_ACK else
				'1' when READ_STATE = WAIT_ACK else '0';

	RESET_OUTn <= '0' when BOOT_STATE = FLASH_ERASE or BOOT_STATE = ADR_REQ else
				  '0' when BOOT_STATE = BOOT_WR or BOOT_STATE = BOOT_RD else '1'; -- System reset during boot sequence.
	FLASH_RESETn <= RESET_INn when BOOT_STATE = IDLE else
					'0' when WRITE_STATE = WRITE_END else '1';
	FLASH_WEn <= '0' when WRITE_STATE = WRITE_CMD else
				 '0' when WRITE_STATE = WRITE_DATA else '1'; -- Do never write in normal operation.
	FLASH_OEn <= '0' when BOOT_STATE = IDLE else -- Always enabled in normal operation.
				 '0' when BOOT_STATE = BOOT_RD else '1';
	FLASH_CEn <= ROM_CEn when BOOT_STATE = IDLE else
				 '0' when BOOT_STATE = FLASH_ERASE else
				 '0' when BOOT_STATE = BOOT_WR else
				 '0' when BOOT_STATE = BOOT_RD else '1';
end architecture BEHAVIOR;
